static char *cvs_id =" $Id: 7c_cry.c,v 1.1.1.1 2002/01/03 14:52:03 berar Exp $ ";
/*   7c_cry.c     functions to be used on d2am 7circles goniometer  */
/*                this contains crystallographic related functions  */

char *date_7c_cry = __DATE__  ;
char *time_7c_cry = __TIME__  ;

#define EXTERN
#include "7c_def.h"

/* ------------------------------------------------------*/
#define DEBUG(order)             /* order     */
#define VEC_DEBUG(name, vect)    /* put_vector(name, vect); */
#define HKL_DEBUG(name, vect)     put_vector(name, vect);
/* ------------------------------------------------------*/
/* private functions  */
#define EPSILON   1.0e-20
#define EPS_ERR   5.0e-5

static valid_type bragg_angle( hkl_type * hkl, vector_type * gonio );
static valid_type mu_from_nu( hkl_type * hkl );
static valid_type delta_s(vector_type *v, double mu, double nu);
static double angle_hkl( vector_type *a, vector_type *b );
static valid_type solve_mode0(hkl_type * hkl, vector_type * gonio);
static valid_type solve_mode2(hkl_type * hkl, vector_type * gonio);
static valid_type solve_mode3(hkl_type * hkl, vector_type * gonio);

static valid_type angles_to_head(vector_type * w);
static void cell_from_g(cell_type * cell, matrix_type * mat );
static void g_from_cell(matrix_type * mat, cell_type  * cell );
static void head2gonio(vector_type *gonio, vector_type *head);
static void gonio2head(vector_type *head, vector_type *gonio);
static void geom_to_deg(geom_type * geometry );
static void geom_to_rad(geom_type * geometry );
static void bragg_to_deg(bragg_type * brg );
/* ------------------------------------------------------*/
/* all rotation angles use direct rotations except       */
/*     chi which is a reverse rotation around x          */
/* the transformation is done each time chi is called    */
/* ------------------------------------------------------*/
/* this function calculates the diffraction angles :     */
/* the structure hkl is filled with values               */
/* according to the UB matrix, hkl from hkl_cryst and    */       
/* angles choozen in goniometer depending on spec_mode   */
/* ------------------------------------------------------*/
void get_angles(void)
{   
	 vector_type  v, gonio;
    geom_to_rad( & goniometer);
    hkl.ok = INVALID;
    hkl.hkl = hkl_cryst;
    VEC_DEBUG("hkl_cryst", &hkl_cryst)
    mul_matvec( &v, &mat_ub, &hkl_cryst);
    VEC_DEBUG("head", &v)
    head2gonio( &gonio, &v);
    VEC_DEBUG("gonio", &gonio)
    hkl.ok = bragg_angle( & hkl, & gonio) ;
    if ( hkl.ok == OK ) 
    {  hkl.bragg.pos.alpha = hkl.bragg.pos.beta = hkl.bragg.pos.azimuth = 0;
       hkl.result.pos.rx = 1;
       hkl.result.pos.ry = hkl.result.pos.rz = 0;
       switch ( spec_mode )
       {   case 0 : hkl.result.pos.nu = 0;
                 hkl.result.pos.mu = hkl.bragg.pos.bragg;
                 hkl.result.pos.rho = 0;
                 hkl.result.pos.omega = hkl.bragg.pos.theta;
                 hkl.ok = solve_mode0 ( & hkl, &gonio );
                 hkl.bragg.pos.azimuth = angle_hkl( &hkl.hkl, &azimuth );
                 break;
           case 1 : hkl.result.pos.nu = goniometer.pos.nu;
                 hkl.ok = mu_from_nu (&hkl) ;
                 if ( hkl.ok != OK ) return;
                 hkl.result.pos.rho = 0;
                 hkl.result.pos.omega = goniometer.pos.omega;
                 hkl.ok = solve_mode0 ( & hkl, &gonio );
                 hkl.bragg.pos.azimuth = angle_hkl( &hkl.hkl, &azimuth );
                 break;
           case 2 : hkl.result.pos.nu = goniometer.pos.nu;
                 hkl.ok = mu_from_nu (&hkl) ;
                 if ( hkl.ok != OK ) return;
                 hkl.result.pos.rho = 0;
                 hkl.result.pos.chi = goniometer.pos.chi;
                 hkl.result.pos.phi = goniometer.pos.phi;
                 hkl.ok = solve_mode2 ( & hkl, &gonio );
                 hkl.bragg.pos.azimuth = angle_hkl( &hkl.hkl, &azimuth );
                 break;
           case 3 : hkl.result.pos.nu = goniometer.pos.nu;
                 hkl.ok = mu_from_nu (&hkl) ;
                 if ( hkl.ok != OK ) return;
                 hkl.result.pos.rho = 0;
                 hkl.result.pos.phi = goniometer.pos.phi;
                 hkl.ok = solve_mode3 ( & hkl, &gonio );
                 hkl.bragg.pos.azimuth = angle_hkl( &hkl.hkl, &azimuth );
                 break;
           case 4 : hkl.result.pos.nu = goniometer.pos.nu;
                 hkl.ok = mu_from_nu (&hkl) ;
                 if ( hkl.ok != OK ) return;
                 hkl.result.pos.rho = 0;
                 hkl.result.pos.phi = goniometer.pos.phi;
                 hkl.ok = solve_mode3 ( & hkl, &gonio );
                 hkl.bragg.pos.azimuth = angle_hkl( &hkl.hkl, &azimuth );
                 break;
           }
       }
    geom_to_deg( & goniometer);
    geom_to_deg( & hkl.result);
    bragg_to_deg(& hkl.bragg);
    }
/* ------------------------------------------------------*/
/* ------------------------------------------------------*/
/* this function calulates h, k ,l                       */
/* fills the structure hkl with values                   */
/* according to the UB matrix and angles in goniometer   */       
/* ------------------------------------------------------*/
void get_hkl(void)
{   
	 vector_type  w;
    
	 if ( angles_to_head( &w ) == INVALID ) return;
    VEC_DEBUG("head", &w)
	 mul_matvec( & hkl.hkl, & mat_inv_ub, &w );
    VEC_DEBUG("hkl", &hkl.hkl)
    hkl.ok = bragg_angle( & hkl, &w) ;
    geom_to_deg( &hkl.result);
    hkl.bragg.pos.azimuth = angle_hkl( &hkl.hkl, &azimuth );
    bragg_to_deg( &hkl.bragg);
    }
/* ------------------------------------------------------*/
/* ------------------------------------------------------*/
/* this function calulates the UB matrix                 */
/* from data in or0 and or1                            */
/* according to the known lattice constant               */ 
/* calculation of the componants of 3 vectors on head    */
/* and on the recip base and solve HEAD = UB . HKL       */
/* ------------------------------------------------------*/
void ub_from_or(void)
{   
    hkl_type or2;
    double n;
	 matrix_type ga, gb;
	 matrix_type head, inv_index, index;
    
    /* head coordinates of the two vectors */
    goniometer= or0.result;
printf("or0  gonio = %8.4lf  %8.4lf  %8.4lf  %8.4lf\n",
             goniometer.pos.rho, goniometer.pos.omega, 
             goniometer.pos.chi, goniometer.pos.phi);
printf("or0  detec = %8.4lf  %8.4lf  %6.1lf %6.1lf %6.1lf\n",
             goniometer.pos.mu, goniometer.pos.nu, 
             goniometer.pos.rx, goniometer.pos.ry, goniometer.pos.rz);
	 if ( angles_to_head( &head.v[0] ) == INVALID ) return;
printf("===>head[0] %lf %lf %lf\n",head.v[0].coord.x, head.v[0].coord.y, head.v[0].coord.z);
    goniometer= or1.result;
printf("or1   gonio = %8.4lf  %8.4lf  %8.4lf  %8.4lf\n",
             goniometer.pos.rho, goniometer.pos.omega, 
             goniometer.pos.chi, goniometer.pos.phi);
printf("or1   detec = %8.4lf  %8.4lf   %6.1lf %6.1lf %6.1lf\n",
             goniometer.pos.mu, goniometer.pos.nu, 
             goniometer.pos.rx, goniometer.pos.ry, goniometer.pos.rz);
    if ( angles_to_head( &head.v[1] ) == INVALID ) return;
printf("===>head[1] %lf %lf %lf\n",head.v[1].coord.x, head.v[1].coord.y, head.v[1].coord.z);
    /* calc the metrix tensor from cell constants */
	 g_from_cell ( &ga, &direct);
	 inv_mat (&gb, &ga);
printf("===>gb %lf %lf %lf\n     %lf %lf %lf\n     %lf %lf %lf\n",
   gb.m[0][0], gb.m[0][1], gb.m[0][2],
   gb.m[1][0], gb.m[1][1], gb.m[1][2],
   gb.m[2][0], gb.m[2][1], gb.m[2][2]);
    /* hkl of the product of the two vectors on recip base*/
printf("===>or0 %lf %lf %lf\n",
or0.hkl.coord.x, or0.hkl.coord.y, or0.hkl.coord.z); 
printf("===>or1 %lf %lf %lf\n",
or1.hkl.coord.x, or1.hkl.coord.y, or1.hkl.coord.z); 
   /* calculation of (hok1-h1k0)ar*br */
    n = sqrt(( gb.m[0][0]*gb.m[1][1]-gb.m[0][1]*gb.m[1][0])/gb.m[2][2]);
    n *= or0.hkl.coord.x*or1.hkl.coord.y - or1.hkl.coord.x*or0.hkl.coord.y;
    or2.hkl.coord.x = -n * gb.m[0][2]/gb.m[0][0];
    or2.hkl.coord.y = -n * gb.m[1][2]/gb.m[1][1];
    or2.hkl.coord.z = n;
    /* calculation of (k0l1-k1l0)br*cr */
    n = sqrt(( gb.m[1][1]*gb.m[2][2]-gb.m[1][2]*gb.m[2][1])/gb.m[0][0]);
    n *= or0.hkl.coord.y*or1.hkl.coord.z - or1.hkl.coord.y*or0.hkl.coord.z;
    or2.hkl.coord.x += n;
    or2.hkl.coord.y -= n * gb.m[1][0]/gb.m[1][1];
    or2.hkl.coord.z -= n * gb.m[2][0]/gb.m[2][2];
     /* calculation of (l0h1-l1h0)cr*ar */
    n = sqrt(( gb.m[0][0]*gb.m[2][2]-gb.m[0][2]*gb.m[2][0])/gb.m[1][1]);
    n *= or0.hkl.coord.z*or1.hkl.coord.x - or1.hkl.coord.z*or0.hkl.coord.x;
    or2.hkl.coord.x -= n * gb.m[0][1]/gb.m[0][0];
    or2.hkl.coord.y += n;
    or2.hkl.coord.z -= n * gb.m[2][1]/gb.m[2][2];
printf("===>or2 %lf %lf %lf\n",
or2.hkl.coord.x, or2.hkl.coord.y, or2.hkl.coord.z); 
    /* hkl of the product of the two vectors using head values*/
    head.v[2].coord.x = head.v[0].coord.y * head.v[1].coord.z - head.v[0].coord.z * head.v[1].coord.y;
    head.v[2].coord.y = head.v[0].coord.z * head.v[1].coord.x - head.v[0].coord.x * head.v[1].coord.z;
    head.v[2].coord.z = head.v[0].coord.x * head.v[1].coord.y - head.v[0].coord.y * head.v[1].coord.x;
printf("===>head[2] %lf %lf %lf\n",head.v[2].coord.x, head.v[2].coord.y, head.v[2].coord.z);
    /* the matrix of the 3 vectors in recip base */
     if ( spec_ub == 1) 
     {   or2.hkl.coord.x /= M_2PI; 
         or2.hkl.coord.y /= M_2PI; 
         or2.hkl.coord.z /= M_2PI; 
         } 
	 index.v[0]=or0.hkl;
    index.v[1]=or1.hkl;
    index.v[2]=or2.hkl;
	 if ( inv_mat( &inv_index, &index) ) return;
    mul_matmat( &mat_ub, &head, &inv_index );
	 inv_mat (&mat_inv_ub, &mat_ub);
	 }
/* ------------------------------------------------------*/
/* ------------------------------------------------------*/
/* filled the structure recip and direct                 */
/* ------------------------------------------------------*/
void cell_from_ub(void)
{    int i, j;
	  matrix_type ga, gb;
     for ( i = 0; i < 3; i++ ) for ( j = 0; j < 3; j++ ) 
		  gb.m[i][j] = mat_ub.m[0][i] * mat_ub.m[0][j]
						 + mat_ub.m[1][i] * mat_ub.m[1][j]
						 + mat_ub.m[2][i] * mat_ub.m[2][j] ;
     cell_from_g( &recip, &gb);
	  inv_mat ( &ga, &gb );
     cell_from_g( &direct, &ga );
     if ( spec_ub == 1) 
     {   direct.a *= M_2PI; 
         direct.b *= M_2PI; 
         direct.c *= M_2PI;
         } 
     }
/* ------------------------------------------------------*/
/* ------------------------------------------------------*/
/* ------------------------------------------------------*/
char *mode_msg( int mode)
{   switch (mode)
{   case 0 : return "goniometer symetrical mode (rho = nu = 0)";
    case 1 : return "goniometer mode with omega, rho, nu fixed";
    case 2 : return "goniometer mode with chi, phi rho, nu fixed = 'zone mode'";
    case 3 : return "goniometer mode with phi, rho, nu fixed";
    case 4 : return "goniometer mode azimuth, rho and nu fixed";
    default : return " unknown calculation mode";
    }
    }
/* ------------------------------------------------------*/
/* ------------------------------------------------------*/
/*   Following functions are static for internal use     */
/*           crystallographic functions                  */
/* ------------------------------------------------------*/
/* return head coordinates from the goniometer angles    */
/* ------------------------------------------------------*/
static valid_type angles_to_head(vector_type * w)
{
	 int i;
	 vector_type  v;
    double n;
    geom_to_rad( &goniometer);
    hkl.ok = INVALID;
    hkl.result = goniometer;
    /* calculation of the diffracted vector S */
    if (goniometer.pos.rx > EPS_ERR)
    {    v.coord.x = 1.0;
         v.coord.y = goniometer.pos.ry / goniometer.pos.rx;
         v.coord.z = goniometer.pos.rz / goniometer.pos.rx;
         n = sqrt( 1 + v.coord.y * v.coord.y + v.coord.z * v.coord.z ); 
         for (i = 0; i < 3; i++ ) v.v[i] /= n;
         if ( spec_ub == 1) for (i = 0; i < 3; i++ ) v.v[i] *=M_2PI;
         }
    else 
    {    v.coord.x = ( spec_ub == 1) ? M_2PI : 1.0;
         v.coord.y = v.coord.z = 0;
         }
    rotate(2, w, goniometer.pos.nu, &v);
    rotate(1, &v, goniometer.pos.mu, w);
    /* calculation of  S - S0 */
    v.coord.x -= ( spec_ub == 1) ? M_2PI : 1.0;
    if ( lambda <= EPS_ERR ) { 
        if ( p_errormsg ) fprintf(stderr,"ERROR get_hkl : invalid lambda value %lg\n", lambda);
        hkl.ok = INVALID; 
        return INVALID; 
        }
    for ( i = 0; i <3 ; i++) v.v[i] *= 1/lambda;
    VEC_DEBUG("s   ", &v)
    /* calculation of  the vector hkl from the lab to the crystal */
    rotate(2, w, -goniometer.pos.rho, &v);
    rotate(1, &v, -goniometer.pos.omega, w);
    rotate(0, w, goniometer.pos.chi, &v);  /* chi is reverse */
    rotate(1, &v, -goniometer.pos.phi, w);
    geom_to_deg( &goniometer);
    VEC_DEBUG("gonio", w)
    gonio2head( w, &v );
    VEC_DEBUG("head", &v)
    return OK;
    }
/* ------------------------------------------------------*/
/* filled hkl with scattering angles psi and theta       */
/* ------------------------------------------------------*/
static valid_type bragg_angle( hkl_type * hkl, vector_type * gonio )
{   double q;
    q = gonio->coord.x * gonio->coord.x + gonio->coord.y * gonio->coord.y 
      + gonio->coord.z * gonio->coord.z ;
    q = 0.5 * lambda * sqrt(q);
    if ( spec_ub == 1) q /= M_2PI;
    if ( q >= 1) 
    {   if (p_errormsg ) fprintf(stderr,"ERROR bragg_angle: out of q_range %lf\n", q);
        return Q_RANGE;
        }
    hkl->bragg.pos.bragg = 2 * ( hkl->bragg.pos.theta = asin(q));
    return OK; 
    }
/* ------------------------------------------------------*/
/* ------------------------------------------------------*/
/* calculated the detector vertical angle                */
/* ------------------------------------------------------*/
static valid_type mu_from_nu( hkl_type * hkl )
{   double  r = cos(hkl->bragg.pos.bragg) / cos(hkl->result.pos.nu);
    if ( fabs(r) > 1.0 )
    {   if (p_errormsg ) fprintf(stderr,"ERROR mu_from_nu: out of det_range %lf\n", r);
        return DET_RANGE;
        }
    hkl->result.pos.mu =acos(r);
    return OK;
    }
/* ------------------------------------------------------*/
/* ------------------------------------------------------*/
/* diff between incident beam and scattered vector       */
/* ------------------------------------------------------*/
static valid_type delta_s( vector_type *v, double mu, double nu )
{   int i;
	 vector_type w;
    /* the diffracted vector S */
    v->coord.x = ( spec_ub == 1 ) ? M_2PI : 1.0;
    v->coord.y = v->coord.z = 0;
    rotate(2, &w, nu, v);
    rotate(1, v, mu, &w);
    v->coord.x -= ( spec_ub == 1 ) ? M_2PI : 1.0;
    if (lambda < EPS_ERR )
    {   if ( p_errormsg ) fprintf(stderr,"ERROR get_hkl, invalid lambda %lf\n", lambda);
        return INVALID;
        }
    for (i=0; i <3; i++) v->v[i] /= lambda;
	 return OK;
    }
/* ------------------------------------------------------*/
/* ------------------------------------------------------*/
/* cell parameter from the metric tensor                 */
/* ------------------------------------------------------*/
static void cell_from_g( cell_type * cell, matrix_type  *mat )
{
     cell->a = sqrt( mat->m[0][0] );       
	  cell->b = sqrt( mat->m[1][1] );
     cell->c = sqrt( mat->m[2][2] );       
     cell->alpha = TO_DEG * acos( mat->m[1][2] / cell->b / cell->c );       
     cell->beta  = TO_DEG * acos( mat->m[2][0] / cell->c / cell->a );       
     cell->gamma = TO_DEG * acos( mat->m[0][1] / cell->a / cell->b );       
     }
/* ------------------------------------------------------*/
/* ------------------------------------------------------*/
/* metric tensor from cell parameter                 */
/* ------------------------------------------------------*/
static void g_from_cell( matrix_type  *mat, cell_type * cell)
{
     mat->m[0][0] = cell->a * cell->a ;       
     mat->m[1][1] = cell->b * cell->b ;       
     mat->m[2][2] = cell->c * cell->c ;       
     mat->m[0][1] = mat->m[1][0] = cell->a * cell->a * cos (cell->alpha / TO_DEG);       
     mat->m[0][2] = mat->m[2][0] = cell->a * cell->c * cos (cell->beta / TO_DEG);       
     mat->m[1][2] = mat->m[2][1] = cell->b * cell->c * cos (cell->gamma / TO_DEG);       
     }
/* ------------------------------------------------------*/
/* ------------------------------------------------------*/
/* angle between two h, k, l vectors                     */
/* ------------------------------------------------------*/
static double angle_hkl( vector_type *a, vector_type *b )
{    vector_type oa, ob;
     double na, nb, c;
     mul_matvec ( &oa, &mat_ub, a);
	  mul_matvec ( &ob, &mat_ub, b);
     na = a->coord.x * a->coord.x + a->coord.y * a->coord.y + a->coord.z * a->coord.z;
     nb = b->coord.x * b->coord.x + b->coord.y * b->coord.y + b->coord.z * b->coord.z;
     c = a->coord.x * b->coord.x + a->coord.y * b->coord.y + a->coord.z * b->coord.z;
     return acos(c / sqrt ( na *nb ));
     }
/* ------------------------------------------------------*/
/* ------------------------------------------------------*/
/* crystal angles calculation for spec_mode 0 or 1       */
/* rho, omega, mu, nu  are fixed                         */
/* ------------------------------------------------------*/
static valid_type solve_mode0(hkl_type * hkl, vector_type * gonio)
{    vector_type v, w;
     double phi[2], chi[2];
     int ok[2], i, m, n_sol;
     hkl->ok = delta_s( &v, hkl->result.pos.mu, hkl->result.pos.nu );
     if ( hkl->ok != OK ) return hkl->ok;
     rotate(2, &w, -hkl->result.pos.rho, &v );
     rotate(1, &v, -hkl->result.pos.omega, &w );
     VEC_DEBUG("solve_mode0 : gonio", gonio)
     n_sol = solve_trigo(phi, -gonio->coord.x, gonio->coord.z, v.coord.x);
     for ( i = m = 0; i < n_sol; i++)
     {    
     DEBUG(printf(" sol %d from solve trigo %lf\n", i, phi[i]*TO_DEG);)
	  rotate(1, &w, phi[i], gonio );
          ok[i]=solve_rot(0, &chi[i], &w, &v ); 
          if ( ok[i] == 0 ) continue; 
     DEBUG(printf( " %d ... %lf   %lf\n", i, phi[i]*TO_DEG, chi[i]*TO_DEG);)
          m++;
          }
     if ( m == 0 )   return MODE_RANGE;
     if ( m == 2 )
     {  if ( fabs(phi[0]) <= fabs(phi[1]) && fabs(chi[0]) <= fabs(chi[1]) )
        {   hkl->result.pos.phi = phi[0];
            hkl->result.pos.chi = -chi[0];
            }
        else
        {   hkl->result.pos.phi = phi[1];
            hkl->result.pos.chi = -chi[1];
            }
        return OK;
        }   
     if ( ok[0] == 1 )
     {   hkl->result.pos.phi = phi[0];
         hkl->result.pos.chi = -chi[0];
         }
     else
     {   hkl->result.pos.phi = phi[1];
         hkl->result.pos.chi = -chi[1];
         }
     return OK;
     }    
/* ------------------------------------------------------*/
/* crystal angles calculation for spec_mode 2            */
/* all are fixed except omega                            */
/* ------------------------------------------------------*/
static valid_type solve_mode2(hkl_type * hkl, vector_type * gonio)
{    vector_type u, v, w;
     double omeg;
     hkl->ok = delta_s( &v, hkl->result.pos.mu, hkl->result.pos.nu );
     if ( hkl->ok != OK ) return hkl->ok;
     rotate(2, &w, -hkl->result.pos.rho, &v );
     rotate(1, &v, hkl->result.pos.phi, gonio );
     rotate(0, &u, -hkl->result.pos.chi, &v );
     if ( solve_rot(1, &omeg, &w, &u ) == 0 ) return MODE_RANGE;
     hkl->result.pos.omega = -omeg;
     return OK;
     }    
/* ------------------------------------------------------*/
/* crystal angles calculation for spec_mode 3            */
/* omega and chi are calculated                          */
/* ------------------------------------------------------*/
static valid_type solve_mode3(hkl_type * hkl, vector_type * gonio)
{    vector_type u, v, w;
     double ome[2], chi[2];
     int ok[2], i, m, n_sol;
     hkl->ok = delta_s( &v, hkl->result.pos.mu, hkl->result.pos.nu );
     if ( hkl->ok != OK ) return hkl->ok;
     rotate(2, &w, -hkl->result.pos.rho, &v );
     rotate(1, &v, hkl->result.pos.phi, gonio );
     n_sol = solve_trigo(ome, -w.coord.x, w.coord.z, v.coord.x);
     for ( i = m = 0; i < n_sol; i++)
     {    
     DEBUG(printf(" sol %d from solve trigo %lf\n", i, ome[i]*TO_DEG);)
          rotate(1, &u, ome[i], &w );
          ok[i]=solve_rot(0, &chi[i], &v, &u );
          if ( ok[i] == 0 ) continue; 
     DEBUG(printf( " %d ... %lf   %lf\n", i, chi[i]*TO_DEG, ome[i]*TO_DEG);)
          m++;
          }
     if ( m == 0 )   return MODE_RANGE;
     if ( m == 2 )
     {  if ( fabs(ome[1]) > fabs(ome[0]) && fabs(chi[1]) > fabs(chi[0]) )
        {   hkl->result.pos.omega = -ome[0];
            hkl->result.pos.chi = -chi[0];
            }
        else
        {   hkl->result.pos.omega = -ome[1];
            hkl->result.pos.chi = -chi[1];
            }
        return OK;
        }   
     if ( ok[0] == 1 )
     {   hkl->result.pos.omega = -ome[0];
         hkl->result.pos.chi = -chi[0];
         }
     else
     {   hkl->result.pos.omega = -ome[1];
         hkl->result.pos.chi = -chi[1];
         }
     return OK;
     }    
/* ------------------------------------------------------*/
/* ------------------------------------------------------*/
/*  transform coordinates from gonio ref to the head(UB) */
/* ------------------------------------------------------*/
static void head2gonio( vector_type *gonio, vector_type *head)
{   gonio->coord.x= head->coord.y;
    gonio->coord.y= head->coord.z;
    gonio->coord.z= head->coord.x;
    }
static void gonio2head( vector_type *head, vector_type *gonio)
{   head->coord.x= gonio->coord.z;
    head->coord.y= gonio->coord.x;
    head->coord.z= gonio->coord.y;
    }
/* ------------------------------------------------------*/
/*  transform angles in rad or degrees                   */
/* ------------------------------------------------------*/
static void geom_to_deg( geom_type * geometry )
{   int i;
    for ( i = 0; i < GEOM-3; i++ ) geometry->angle[i] *= TO_DEG ;
    }
static void geom_to_rad( geom_type * geometry )
{   int i;
    for ( i = 0; i < GEOM-3; i++ ) geometry->angle[i] /= TO_DEG ;
    }
static void bragg_to_deg( bragg_type * brg )
{   int i;
    for ( i = 0; i < 5; i++ ) brg->angle[i] *= TO_DEG ;
    }
/* ------------------------------------------------------*/
